#define _CLOCK_DEBUG(s) asm(s)
//#define _CLOCK_DEBUG(s)

extern void __BadCPUSpeed_in_SimpleClockInit();

static inline void SimpleClockInit(long, long) __attribute__((always_inline));
static inline void SimpleClockInit(long cpuspeed, long xtalspeed)
{
    uint32_t pllcfg;

    if (cpuspeed > 72000000)
	__BadCPUSpeed_in_SimpleClockInit();
    if (xtalspeed == 0) {
	if (cpuspeed == 8000000) {
	    /*
	     * Easy case - run off internal oscillator
	     * (which is the default configuration.)
	     */
	    return;
	} else {
	    _CLOCK_DEBUG("nop; PICK_HSI: nop");
	    xtalspeed = 4000000;
	    pllcfg = RCC_CFGR_PLLSRC_HSI_Div2;
	}
    } else {
	_CLOCK_DEBUG("nop; PICK_HSE:");
	pllcfg = RCC_CFGR_PLLSRC_HSE;
	/*
	 * Start the crystal oscillator, and wait for start.
	 */
	_CLOCK_DEBUG("nop; HSE_ON:");
	RCC->CR |= RCC_CR_HSEON;
	while ((RCC->CR & RCC_CR_HSERDY) == 0)
	    /* spin */ ;
    }

    /*
     * For faster clock speeds, we need to adjust the flash wait-states,
     * and the prescalers for the slower peripheral buses
     */
    if (cpuspeed > 48000000) {
	_CLOCK_DEBUG("Latency2:");
	FLASH->ACR |= FLASH_ACR_LATENCY_2;
    } else if (cpuspeed > 24000000) {
	_CLOCK_DEBUG("Latency1:");
	FLASH->ACR |= FLASH_ACR_LATENCY_1;
    }

    if (cpuspeed > 36000000) {
	RCC->CFGR |= RCC_CFGR_PPRE1_DIV2; /* AHB1 Clock */
    }

    /*
     * Configure and enable the PLL, and wait for it to start
     switch (cpuspeed/xtalspeed) {
     case 1:
     */

    if (cpuspeed == xtalspeed) {
	_CLOCK_DEBUG("USE_HSE:");
	pllcfg |= RCC_CFGR_SW_HSE;
	RCC->CFGR |= pllcfg;
	return; 			/* done without starting PLL */
    } else if (cpuspeed == 2*xtalspeed) {
	_CLOCK_DEBUG("PLL_2:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL2;
    } else if (cpuspeed == 3*xtalspeed) {
	_CLOCK_DEBUG("PLL_3:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL3;
    } else if (cpuspeed == 4*xtalspeed) {
	_CLOCK_DEBUG("PLL_4:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL4;
    } else if (cpuspeed == 5*xtalspeed) {
	_CLOCK_DEBUG("PLL_5:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL5;
    } else if (cpuspeed == 6*xtalspeed) {
	_CLOCK_DEBUG("PLL_6:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL6;
    } else if (cpuspeed == 7*xtalspeed) {
	_CLOCK_DEBUG("PLL_7:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL7;
    } else if (cpuspeed == 8*xtalspeed) {
	_CLOCK_DEBUG("PLL_8:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL8;
    } else if (cpuspeed == 9*xtalspeed) {
	_CLOCK_DEBUG("PLL_9:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL9;
    } else if (cpuspeed == 10*xtalspeed) {
	_CLOCK_DEBUG("PLL_10:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL10;
    } else if (cpuspeed == 11*xtalspeed) {
	_CLOCK_DEBUG("PLL_11:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL11;
    } else if (cpuspeed == 12*xtalspeed) {
	_CLOCK_DEBUG("PLL_12:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL12;
    } else if (cpuspeed == 13*xtalspeed) {
	_CLOCK_DEBUG("PLL_13:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL13;
    } else if (cpuspeed == 14*xtalspeed) {
	_CLOCK_DEBUG("PLL_14:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL14;
    } else if (cpuspeed == 15*xtalspeed) {
	_CLOCK_DEBUG("PLL_15:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL15;
    } else if (cpuspeed == 16*xtalspeed) {
	_CLOCK_DEBUG("PLL_16:");
	pllcfg |= RCC_CFGR_SW_PLL|RCC_CFGR_PLLMULL16;
    } else {
	__BadCPUSpeed_in_SimpleClockInit();
    }
    _CLOCK_DEBUG("PLL_CFGR:");
    RCC->CFGR |= pllcfg;
    _CLOCK_DEBUG("PLL_CR:");
    RCC->CR |= RCC_CR_PLLON;		/* Turn on PLL */
    _CLOCK_DEBUG("PLL_SPIN:");
    while ((RCC->CR & RCC_CR_PLLRDY) == 0)
	/* Spin */ ;

    _CLOCK_DEBUG("USE_PLL:");

    RCC->CFGR |= RCC_CFGR_SW_PLL;	/* Select PLL Clock */
    _CLOCK_DEBUG("Clock_done:");
    
}

